@@ -177,13 +177,13 @@ checker_terminate_phase2(void)
 	ipvs_stop();
 
 	/* Stop daemon */
-	pidfile_rm(checkers_pidfile);
+	pidfile_rm(&checkers_pidfile);
 
 	/* Clean data */
 	if (global_data)
-		free_global_data(global_data);
+		free_global_data(&global_data);
 	if (check_data)
-		free_check_data(check_data);
+		free_check_data(&check_data);
 	free_parent_mallocs_exit();
 
 	/*
@@ -326,8 +326,9 @@ start_check(data_t *prev_global_data)
 		init_global_data(global_data, prev_global_data, true);
 
 	/* Update process name if necessary */
-	if ((!reload && global_data->lvs_process_name) ||
-	    (reload &&
+	if ((!prev_global_data &&		// startup
+	     global_data->lvs_process_name) ||
+	    (prev_global_data &&		// reload
 	     (!global_data->lvs_process_name != !prev_global_data->lvs_process_name ||
 	      (global_data->lvs_process_name && strcmp(global_data->lvs_process_name, prev_global_data->lvs_process_name)))))
 		set_process_name(global_data->lvs_process_name);
@@ -518,8 +519,8 @@ reload_check_thread(__attribute__((unused)) thread_ref_t thread)
 	start_check(old_global_data);
 
 	/* free backup data */
-	free_check_data(old_check_data);
-	free_global_data(old_global_data);
+	free_check_data(&old_check_data);
+	free_global_data(&old_global_data);
 
 #ifndef _ONE_PROCESS_DEBUG_
 	save_config(true, "check", dump_data_check);
@@ -565,7 +566,10 @@ static void
 check_signal_init(void)
 {
 	signal_set(SIGHUP, sigreload_check, NULL);
-	signal_set(SIGINT, sigend_check, NULL);
+	if (ignore_sigint)
+		signal_ignore(SIGINT);
+	else
+		signal_set(SIGINT, sigend_check, NULL);
 	signal_set(SIGTERM, sigend_check, NULL);
 	signal_set(SIGUSR1, sigusr1_check, NULL);
 #ifdef THREAD_DUMP
@@ -707,6 +711,8 @@ start_check_child(void)
 	deregister_thread_addresses();
 #endif
 
+	close_other_pidfiles();
+
 #ifdef _WITH_BFD_
 	/* Close the write end of the BFD checker event notification pipe and the track_process fd */
 	close(bfd_checker_event_pipe[1]);
@@ -755,7 +761,7 @@ start_check_child(void)
 	separate_config_file();
 
 	/* Child process part, write pidfile */
-	if (!pidfile_write(checkers_pidfile, getpid())) {
+	if (!pidfile_write(&checkers_pidfile)) {
 		log_message(LOG_INFO, "Healthcheck child process: cannot write pidfile");
 		exit(KEEPALIVED_EXIT_FATAL);
 	}
