@@ -43,7 +43,6 @@
 #include "main.h"
 #include "global_data.h"
 #include "daemon.h"
-#include "config.h"
 #ifndef _ONE_PROCESS_DEBUG_
 #include "config_notify.h"
 #endif
@@ -83,7 +82,6 @@
 #include "namespaces.h"
 #include "scheduler.h"
 #include "keepalived_netlink.h"
-#include "git-commit.h"
 #if defined THREAD_DUMP || defined _EPOLL_DEBUG_ || defined _EPOLL_THREAD_DUMP_ || defined _SCRIPT_DEBUG_
 #include "scheduler.h"
 #endif
@@ -118,6 +116,9 @@
 #ifdef _USE_SYSTEMD_NOTIFY_
 #include "systemd.h"
 #endif
+#ifdef _WITH_SANITIZER_
+#include "sanitizer.h"
+#endif
 #include "warnings.h"
 
 #define CHILD_WAIT_SECS	5
@@ -148,22 +149,18 @@ static const struct child_term children_term[] = {
 const char *version_string = VERSION_STRING;		/* keepalived version */
 const char *conf_file;					/* Configuration file */
 bool reload;						/* Set during a reload */
-const char *main_pidfile;				/* overrule default pidfile */
-static bool free_main_pidfile;
+struct pidfile main_pidfile = { .fd = -1 };		/* overrule default pidfile */
 #ifdef _WITH_LVS_
 pid_t checkers_child;					/* Healthcheckers child process ID */
-const char *checkers_pidfile;				/* overrule default pidfile */
-static bool free_checkers_pidfile;
+struct pidfile checkers_pidfile = { .fd = -1 };		/* overrule default pidfile */
 #endif
 #ifdef _WITH_VRRP_
 pid_t vrrp_child;					/* VRRP child process ID */
-const char *vrrp_pidfile;				/* overrule default pidfile */
-static bool free_vrrp_pidfile;
+struct pidfile vrrp_pidfile = { .fd = -1 };		/* overrule default pidfile */
 #endif
 #ifdef _WITH_BFD_
 pid_t bfd_child;					/* BFD child process ID */
-const char *bfd_pidfile;				/* overrule default pidfile */
-static bool free_bfd_pidfile;
+struct pidfile bfd_pidfile = { .fd = -1 };		/* overrule default pidfile */
 #endif
 unsigned long daemon_mode;				/* VRRP/CHECK/BFD subsystem selection */
 #ifdef _WITH_SNMP_
@@ -172,6 +169,7 @@ const char *snmp_socket;				/* Socket to use for SNMP agent */
 #endif
 static const char *syslog_ident;			/* syslog ident if not default */
 bool use_pid_dir;					/* Put pid files in /run/keepalived or @localstatedir@/run/keepalived */
+bool children_started;					/* Set once children have been run first time */
 
 unsigned os_major;					/* Kernel version */
 unsigned os_minor;
@@ -211,6 +209,9 @@ static bool create_core_dump = false;
 static const char *core_dump_pattern = "core";
 static char *orig_core_dump_pattern = NULL;
 
+/* Signal handling */
+bool ignore_sigint = false;
+
 /* debug flags */
 #if defined _TIMER_CHECK_ || \
     defined _SMTP_ALERT_DEBUG_ || \
@@ -315,29 +316,11 @@ free_parent_mallocs_startup(bool am_child)
 		free_notify_script(&global_data->startup_script);
 		free_notify_script(&global_data->shutdown_script);
 	}
-
-	if (free_main_pidfile) {
-		FREE_CONST_PTR(main_pidfile);
-		free_main_pidfile = false;
-	}
 }
 
 void
 free_parent_mallocs_exit(void)
 {
-#ifdef _WITH_VRRP_
-	if (free_vrrp_pidfile)
-		FREE_CONST_PTR(vrrp_pidfile);
-#endif
-#ifdef _WITH_LVS_
-	if (free_checkers_pidfile)
-		FREE_CONST_PTR(checkers_pidfile);
-#endif
-#ifdef _WITH_BFD_
-	if (free_bfd_pidfile)
-		FREE_CONST_PTR(bfd_pidfile);
-#endif
-
 	FREE_CONST_PTR(config_id);
 
 #ifdef _REPRODUCIBLE_BUILD_
@@ -510,20 +493,20 @@ stop_keepalived(void)
 
 #ifdef _WITH_VRRP_
 	if (__test_bit(DAEMON_VRRP, &daemon_mode))
-		pidfile_rm(vrrp_pidfile);
+		pidfile_close(&vrrp_pidfile, true);
 #endif
 
 #ifdef _WITH_LVS_
 	if (__test_bit(DAEMON_CHECKERS, &daemon_mode))
-		pidfile_rm(checkers_pidfile);
+		pidfile_close(&checkers_pidfile, true);
 #endif
 
 #ifdef _WITH_BFD_
 	if (__test_bit(DAEMON_BFD, &daemon_mode))
-		pidfile_rm(bfd_pidfile);
+		pidfile_close(&bfd_pidfile, true);
 #endif
 
-	pidfile_rm(main_pidfile);
+	pidfile_rm(&main_pidfile);
 #endif
 }
 
@@ -556,7 +539,8 @@ start_keepalived(__attribute__((unused)) thread_ref_t thread)
 		start_check_child();
 		have_child = true;
 		num_reloading++;
-	}
+	} else
+		pidfile_rm(&checkers_pidfile);
 #endif
 #ifdef _WITH_VRRP_
 	/* start vrrp child */
@@ -564,7 +548,8 @@ start_keepalived(__attribute__((unused)) thread_ref_t thread)
 		start_vrrp_child();
 		have_child = true;
 		num_reloading++;
-	}
+	} else
+		pidfile_rm(&vrrp_pidfile);
 #endif
 #ifdef _WITH_BFD_
 	/* start bfd child */
@@ -572,9 +557,12 @@ start_keepalived(__attribute__((unused)) thread_ref_t thread)
 		start_bfd_child();
 		have_child = true;
 		num_reloading++;
-	}
+	} else
+		pidfile_rm(&bfd_pidfile);
 #endif
 
+	children_started = true;
+
 #ifndef _ONE_PROCESS_DEBUG_
 	/* Do we have a reload file to monitor */
 	if (global_data->reload_time_file)
@@ -843,7 +831,7 @@ static bool reload_config(void)
 
 	if (unsupported_change) {
 		/* We cannot reload the configuration, so continue with the old config */
-		free_global_data (global_data);
+		free_global_data(&global_data);
 		global_data = old_global_data;
 	}
 	else {
@@ -852,7 +840,7 @@ static bool reload_config(void)
 		    (global_data->process_name && strcmp(global_data->process_name, old_global_data->process_name)))
 			set_process_name(global_data->process_name);
 
-		free_global_data (old_global_data);
+		free_global_data(&old_global_data);
 	}
 
 	/* There is no point checking the script security of the
@@ -1289,7 +1277,10 @@ signal_init(void)
 #ifdef _WITH_JSON_
 	signal_set(SIGJSON, propagate_signal, NULL);
 #endif
-	signal_set(SIGINT, sigend, NULL);
+	if (ignore_sigint)
+		signal_ignore(SIGINT);
+	else
+		signal_set(SIGINT, sigend, NULL);
 	signal_set(SIGTERM, sigend, NULL);
 #ifdef THREAD_DUMP
 	signal_set(SIGTDUMP, thread_dump_signal, NULL);
@@ -1886,6 +1877,7 @@ usage(const char *prog)
 	fprintf(stderr, "  -i, --config-id id           Skip any configuration lines beginning '@' that don't match id\n"
 			"                                or any lines beginning @^ that do match.\n"
 			"                                The config-id defaults to the node name if option not used\n");
+	fprintf(stderr, "      --ignore-sigint          ignore SIGINT (default means terminate) - used for debugging with GDB\n");
 	fprintf(stderr, "      --signum=SIGFUNC         Return signal number for STOP, RELOAD, DATA, STATS, STATS_CLEAR"
 #ifdef _WITH_JSON_
 								", JSON"
@@ -2051,6 +2043,7 @@ parse_cmdline(int argc, char **argv)
 		{"signum",		required_argument,	NULL,  4 },
 		{"config-test",		optional_argument,	NULL, 't'},
 		{"config-fd",		required_argument,	NULL,  8 },
+		{"ignore-sigint",	no_argument,		NULL,  9 },
 #ifdef _WITH_PERF_
 		{"perf",		optional_argument,	NULL,  5 },
 #endif
@@ -2246,11 +2239,11 @@ parse_cmdline(int argc, char **argv)
 			break;
 #endif
 		case 'p':
-			main_pidfile = optarg;
+			main_pidfile.path = optarg;
 			break;
 #ifdef _WITH_LVS_
 		case 'c':
-			checkers_pidfile = optarg;
+			checkers_pidfile.path = optarg;
 			break;
 		case 'a':
 			__set_bit(LOG_ADDRESS_CHANGES, &debug);
@@ -2258,12 +2251,12 @@ parse_cmdline(int argc, char **argv)
 #endif
 #ifdef _WITH_VRRP_
 		case 'r':
-			vrrp_pidfile = optarg;
+			vrrp_pidfile.path = optarg;
 			break;
 #endif
 #ifdef _WITH_BFD_
 		case 'b':
-			bfd_pidfile = optarg;
+			bfd_pidfile.path = optarg;
 			break;
 #endif
 #ifdef _WITH_SNMP_
@@ -2278,7 +2271,7 @@ parse_cmdline(int argc, char **argv)
 			set_core_dump_pattern = true;
 			if (optarg && optarg[0])
 				core_dump_pattern = optarg;
-			__fallthrough;
+			/* FALLTHROUGH */
 		case 'm':
 			create_core_dump = true;
 			break;
@@ -2356,6 +2349,9 @@ parse_cmdline(int argc, char **argv)
 		case 8:
 			set_config_fd(atoi(optarg));
 			break;
+		case 9:
+			ignore_sigint = true;
+			break;
 		case '?':
 			if (optopt && argv[curind][1] != '-')
 				fprintf(stderr, "Unknown option -%c\n", optopt);
@@ -2459,6 +2455,10 @@ keepalived_main(int argc, char **argv)
 	}
 #endif
 
+#ifdef _WITH_SANITIZER_
+	sanitizer_init();
+#endif
+
 #ifdef _REPRODUCIBLE_BUILD_
 	char *config_opts_read;
 
@@ -2717,53 +2717,53 @@ keepalived_main(int argc, char **argv)
 
 	if (!__test_bit(CONFIG_TEST_BIT, &debug)) {
 		if (global_data->instance_name) {
-			if (!main_pidfile && (main_pidfile = make_pidfile_name(KEEPALIVED_PID_DIR KEEPALIVED_PID_FILE, global_data->instance_name, PID_EXTENSION)))
-				free_main_pidfile = true;
+			if (!main_pidfile.path && (main_pidfile.path = make_pidfile_name(KEEPALIVED_PID_DIR KEEPALIVED_PID_FILE, global_data->instance_name, PID_EXTENSION)))
+				main_pidfile.free_path = true;
 #ifdef _WITH_LVS_
-			if (!checkers_pidfile && (checkers_pidfile = make_pidfile_name(KEEPALIVED_PID_DIR CHECKERS_PID_FILE, global_data->instance_name, PID_EXTENSION)))
-				free_checkers_pidfile = true;
+			if (!checkers_pidfile.path && (checkers_pidfile.path = make_pidfile_name(KEEPALIVED_PID_DIR CHECKERS_PID_FILE, global_data->instance_name, PID_EXTENSION)))
+				checkers_pidfile.free_path = true;
 #endif
 #ifdef _WITH_VRRP_
-			if (!vrrp_pidfile && (vrrp_pidfile = make_pidfile_name(KEEPALIVED_PID_DIR VRRP_PID_FILE, global_data->instance_name, PID_EXTENSION)))
-				free_vrrp_pidfile = true;
+			if (!vrrp_pidfile.path && (vrrp_pidfile.path = make_pidfile_name(KEEPALIVED_PID_DIR VRRP_PID_FILE, global_data->instance_name, PID_EXTENSION)))
+				vrrp_pidfile.free_path = true;
 #endif
 #ifdef _WITH_BFD_
-			if (!bfd_pidfile && (bfd_pidfile = make_pidfile_name(KEEPALIVED_PID_DIR BFD_PID_FILE, global_data->instance_name, PID_EXTENSION)))
-				free_bfd_pidfile = true;
+			if (!bfd_pidfile.path && (bfd_pidfile.path = make_pidfile_name(KEEPALIVED_PID_DIR BFD_PID_FILE, global_data->instance_name, PID_EXTENSION)))
+				bfd_pidfile.free_path = true;
 #endif
 		}
 
 		if (use_pid_dir) {
-			if (!main_pidfile)
-				main_pidfile = KEEPALIVED_PID_DIR KEEPALIVED_PID_FILE PID_EXTENSION;
+			if (!main_pidfile.path)
+				main_pidfile.path = KEEPALIVED_PID_DIR KEEPALIVED_PID_FILE PID_EXTENSION;
 #ifdef _WITH_LVS_
-			if (!checkers_pidfile)
-				checkers_pidfile = KEEPALIVED_PID_DIR CHECKERS_PID_FILE PID_EXTENSION;
+			if (!checkers_pidfile.path)
+				checkers_pidfile.path = KEEPALIVED_PID_DIR CHECKERS_PID_FILE PID_EXTENSION;
 #endif
 #ifdef _WITH_VRRP_
-			if (!vrrp_pidfile)
-				vrrp_pidfile = KEEPALIVED_PID_DIR VRRP_PID_FILE PID_EXTENSION;
+			if (!vrrp_pidfile.path)
+				vrrp_pidfile.path = KEEPALIVED_PID_DIR VRRP_PID_FILE PID_EXTENSION;
 #endif
 #ifdef _WITH_BFD_
-			if (!bfd_pidfile)
-				bfd_pidfile = KEEPALIVED_PID_DIR BFD_PID_FILE PID_EXTENSION;
+			if (!bfd_pidfile.path)
+				bfd_pidfile.path = KEEPALIVED_PID_DIR BFD_PID_FILE PID_EXTENSION;
 #endif
 		}
 		else
 		{
-			if (!main_pidfile)
-				main_pidfile = RUNSTATEDIR "/" KEEPALIVED_PID_FILE PID_EXTENSION;
+			if (!main_pidfile.path)
+				main_pidfile.path = RUNSTATEDIR "/" KEEPALIVED_PID_FILE PID_EXTENSION;
 #ifdef _WITH_LVS_
-			if (!checkers_pidfile)
-				checkers_pidfile = RUNSTATEDIR "/" CHECKERS_PID_FILE PID_EXTENSION;
+			if (!checkers_pidfile.path)
+				checkers_pidfile.path = RUNSTATEDIR "/" CHECKERS_PID_FILE PID_EXTENSION;
 #endif
 #ifdef _WITH_VRRP_
-			if (!vrrp_pidfile)
-				vrrp_pidfile = RUNSTATEDIR "/" VRRP_PID_FILE PID_EXTENSION;
+			if (!vrrp_pidfile.path)
+				vrrp_pidfile.path = RUNSTATEDIR "/" VRRP_PID_FILE PID_EXTENSION;
 #endif
 #ifdef _WITH_BFD_
-			if (!bfd_pidfile)
-				bfd_pidfile = RUNSTATEDIR "/" BFD_PID_FILE PID_EXTENSION;
+			if (!bfd_pidfile.path)
+				bfd_pidfile.path = RUNSTATEDIR "/" BFD_PID_FILE PID_EXTENSION;
 #endif
 		}
 
@@ -2833,7 +2833,7 @@ keepalived_main(int argc, char **argv)
 	}
 
 	/* write the father's pidfile */
-	if (!pidfile_write(main_pidfile, getpid()))
+	if (!pidfile_write(&main_pidfile))
 		goto end;
 
 	if (!global_data->max_auto_priority)
@@ -2885,7 +2885,8 @@ end:
 			getrusage(RUSAGE_SELF, &usage);
 			getrusage(RUSAGE_CHILDREN, &child_usage);
 
-			log_message(LOG_INFO, "CPU usage (self/children) user: %ld.%6.6ld/%ld.%6.6ld system: %ld.%6.6ld/%ld.%6.6ld",
+			log_message(LOG_INFO, "CPU usage (self/children) user: %" PRI_tv_sec ".%6.6" PRI_tv_usec "/%" PRI_tv_sec ".%6.6" PRI_tv_usec
+					" system: %" PRI_tv_sec ".%6.6" PRI_tv_usec "/%" PRI_tv_sec ".%6.6" PRI_tv_usec,
 					usage.ru_utime.tv_sec, usage.ru_utime.tv_usec, child_usage.ru_utime.tv_sec, child_usage.ru_utime.tv_usec,
 					usage.ru_stime.tv_sec, usage.ru_stime.tv_usec, child_usage.ru_stime.tv_sec, child_usage.ru_stime.tv_usec);
 		}
@@ -2909,7 +2910,7 @@ end:
 
 	free_parent_mallocs_startup(false);
 	free_parent_mallocs_exit();
-	free_global_data(global_data);
+	free_global_data(&global_data);
 
 	closelog();
 
