@@ -150,7 +150,7 @@ typedef struct _data {
 	lvs_flush_t			lvs_flush_on_stop;	/* flush any LVS config at shutdown */
 #endif
 	int				max_auto_priority;
-	long				min_auto_priority_delay;
+	unsigned			min_auto_priority_delay;
 #ifdef _WITH_VRRP_
 	struct sockaddr_in6		vrrp_mcast_group6 __attribute__((aligned(__alignof__(sockaddr_t))));
 	struct sockaddr_in		vrrp_mcast_group4 __attribute__((aligned(__alignof__(sockaddr_t))));
@@ -294,6 +294,10 @@ typedef struct _data {
 #ifdef _WITH_JSON_
 	unsigned			json_version;
 #endif
+#ifdef _WITH_VRRP_
+	const char			*iproute_usr_dir;
+	const char			*iproute_etc_dir;
+#endif
 } data_t;
 
 /* Global vars exported */
@@ -305,7 +309,7 @@ extern const char * format_email_addr(const char *);
 extern void alloc_email(const char *);
 extern data_t *alloc_global_data(void);
 extern void init_global_data(data_t *, data_t *, bool);
-extern void free_global_data(data_t *);
+extern void free_global_data(data_t **);
 extern FILE *open_dump_file(const char *) __attribute__((malloc));
 extern void dump_global_data(FILE *, data_t *);
 
