@@ -40,10 +40,8 @@
 
 /* global defs */
 #define SMTP_PORT_STR		"25"
-#define SMTP_BUFFER_LENGTH	512U
-#define SMTP_BUFFER_MAX		1024U
 
-/* SMTP command stage. This values are used along with the enum connect_result
+/* SMTP command stage. These values are used along with the enum connect_result
  * values in the SMTP FSM, and so need to follow them. */
 enum smtp_cmd_state {
 	HELO = connect_result_next,
@@ -52,8 +50,7 @@ enum smtp_cmd_state {
 	DATA,
 	BODY,
 	QUIT,
-	END,
-	ERROR
+	END
 };
 #define SMTP_MAX_FSM_STATE	END
 
@@ -81,35 +78,22 @@ do {				\
     (*(SMTP_FSM[S].send)) (T);	\
 } while (0)
 
-#define SMTP_FSM_READ(S, T, N)		\
+#define SMTP_FSM_READ(S, T)		\
 do {					\
   if ((*(SMTP_FSM[S].read)))		\
-    (*(SMTP_FSM[S].read)) (T, N);	\
+    (*(SMTP_FSM[S].read)) (T);	\
 } while (0)
 
 /* SMTP thread arguments */
 typedef struct _smtp {
-	int		fd;
 	int		stage;
 	email_t		*next_email_element;
 	char		*subject;
 	char		*body;
 	char		*buffer;
-	char		*email_to;
 	size_t		buflen;
 } smtp_t;
 
-/* SMTP command string processing */
-#define SMTP_HELO_CMD    "HELO %s\r\n"
-#define SMTP_MAIL_CMD    "MAIL FROM:<%s>\r\n"
-#define SMTP_RCPT_CMD    "RCPT TO:<%s>\r\n"
-#define SMTP_DATA_CMD    "DATA\r\n"
-#define SMTP_HEADERS_CMD "Date: %s\r\nFrom: %s\r\nSubject: %s\r\n" \
-			 "X-Mailer: Keepalived\r\nTo: %s\r\n\r\n"
-#define SMTP_BODY_CMD    "%s\r\n"
-#define SMTP_SEND_CMD    "\r\n.\r\n"
-#define SMTP_QUIT_CMD    "QUIT\r\n"
-
 #define FMT_SMTP_HOST()	inet_sockaddrtopair(&global_data->smtp_server)
 
 #ifdef _WITH_LVS_
