@@ -65,6 +65,8 @@
 #ifdef _WITH_LVS_
 #include "ipvswrapper.h"
 #endif
+#include "keepalived_netlink.h"
+
 
 /* For load testing recvmsg() */
 /* #define DEBUG_RECVMSG */
@@ -75,8 +77,6 @@
 #endif
 
 /* global vars */
-timeval_t garp_next_time;
-thread_ref_t garp_thread;
 bool vrrp_initialised;
 timeval_t vrrp_delayed_start_time;
 
@@ -266,7 +266,9 @@ vrrp_init_state(list_head_t *l)
 #endif
 
 			/* Set interface state */
-			vrrp_restore_interface(vrrp, false, false);
+			netlink_error_ignore = ESRCH;		// returned if route does not exist
+			vrrp_restore_interface(vrrp, false, true);
+			netlink_error_ignore = 0;
 			if (is_up &&
 			    new_state != VRRP_STATE_FAULT &&
 			    !vrrp->num_script_init &&
@@ -660,6 +662,31 @@ vrrp_lower_prio_gratuitous_arp_thread(thread_ref_t thread)
 	vrrp_send_link_update(vrrp, vrrp->garp_lower_prio_rep);
 }
 
+/* Gratuitous ARP refresh thread (i.e. periodic send of GARP messages) */
+void
+vrrp_gratuitous_arp_refresh_thread(thread_ref_t thread)
+{
+	vrrp_t *vrrp = THREAD_ARG(thread);
+
+	vrrp_send_link_update(vrrp, vrrp->garp_refresh_rep);
+	thread_add_timer(master, vrrp_gratuitous_arp_refresh_thread,
+			 vrrp, timer_long(vrrp->garp_refresh));
+}
+
+#ifdef _HAVE_VRRP_VMAC_
+/* Gratuitous ARP VMAC update thread (i.e. one GARP per VMAC interface
+ * on which VIPs are configured. */
+void
+vrrp_gratuitous_arp_vmac_update_thread(thread_ref_t thread)
+{
+	vrrp_t *vrrp = THREAD_ARG(thread);
+
+	vrrp_send_vmac_update(vrrp);
+	thread_add_timer(master, vrrp_gratuitous_arp_vmac_update_thread,
+			 vrrp, timer_long(vrrp->vmac_garp_intvl));
+}
+#endif
+
 void
 try_up_instance(vrrp_t *vrrp, bool leaving_init)
 {
@@ -1091,18 +1118,18 @@ vrrp_dispatcher_read(sock_t *sock)
 				expected_cmsg = true;
 				if (cmsg->cmsg_type == SO_TIMESTAMPNS) {
 					strftime(time_buf, sizeof time_buf, "%T", localtime(&ts->tv_sec));
-					log_message(LOG_INFO, "TIMESTAMPNS (socket %d - VRID %u) %s.%9.9ld"
+					log_message(LOG_INFO, "TIMESTAMPNS (socket %d - VRID %u) %s.%9.9" PRI_ts_nsec
 							    , sock->fd_in, hd->vrid, time_buf, ts->tv_nsec);
 				}
 #if 0
 				if (cmsg->cmsg_type == SO_TIMESTAMP) {
 					struct timeval *tv = (void *)CMSG_DATA(cmsg);
-					log_message(LOG_INFO, "TIMESTAMP message (%d - %u)  %ld.%9.9ld"
+					log_message(LOG_INFO, "TIMESTAMP message (%d - %u)  %" PRI_tv_sec ".%6.6" PRI_tv_usec
 							    , sock->fd_in, hd->vrid, tv->tv_sec, tv->tv_usec);
 				}
 				else if (cmsg->cmsg_type == SO_TIMESTAMPING) {
 					struct timespec *ts = (void *)CMSG_DATA(cmsg);
-					log_message(LOG_INFO, "TIMESTAMPING message (%d - %u)  %ld.%9.9ld, raw %ld.%9.9ld"
+					log_message(LOG_INFO, "TIMESTAMPING message (%d - %u)  %" PRI_ts_sec ".%9.9" PRI_ts_nsec ", raw %" PRI_ts_sec ".%9.9" PRI_ts_nsec
 							    , sock->fd_in, hd->vrid, ts->tv_sec, ts->tv_nsec, (ts+2)->tv_sec, (ts+2)->tv_nsec);
 				}
 #endif
@@ -1377,88 +1404,54 @@ vrrp_script_child_thread(thread_ref_t thread)
 	vscript->init_state = SCRIPT_INIT_STATE_DONE;
 }
 
-/* Delayed ARP/NA thread */
-static int
-vrrp_arpna_send(vrrp_t *vrrp, list_head_t *l, timeval_t *n)
-{
-	ip_address_t *ip_addr;
-	interface_t *ifp;
-
-	list_for_each_entry(ip_addr, l, e_list) {
-		if (!ip_addr->garp_gna_pending)
-			continue;
-
-		if (!ip_addr->set) {
-			ip_addr->garp_gna_pending = false;
-			continue;
-		}
-
-		ifp = IF_BASE_IFP(ip_addr->ifp);
-
-		/* This should never happen */
-		if (!ifp->garp_delay) {
-			ip_addr->garp_gna_pending = false;
-			continue;
-		}
-
-		/* IPv4 handling */
-		if (!IP_IS6(ip_addr)) {
-			if (timercmp(&time_now, &ifp->garp_delay->garp_next_time, >=)) {
-				send_gratuitous_arp_immediate(ifp, ip_addr);
-				ip_addr->garp_gna_pending = false;
-			} else {
-				vrrp->garp_pending = true;
-				if (timercmp(&ifp->garp_delay->garp_next_time, n, <))
-					*n = ifp->garp_delay->garp_next_time;
-			}
-			continue;
-		}
-
-		/* IPv6 handling */
-		if (timercmp(&time_now, &ifp->garp_delay->gna_next_time, >=)) {
-			ndisc_send_unsolicited_na_immediate(ifp, ip_addr);
-			ip_addr->garp_gna_pending = false;
-		} else {
-			vrrp->gna_pending = true;
-			if (timercmp(&ifp->garp_delay->gna_next_time, n, <))
-				*n = ifp->garp_delay->gna_next_time;
-		}
-	}
-
-	return 0;
-}
-
+/* Thread to send gratuitous ARPs when the sending is rate limited */
 void
 vrrp_arp_thread(thread_ref_t thread)
 {
-	vrrp_t *vrrp;
-	timeval_t next_time = {
-		.tv_sec = INT_MAX	/* We're never going to delay this long - I hope! */
-	};
+	ip_address_t *ip_addr;
+	interface_t *ifp = THREAD_ARG(thread);
 
-	set_time_now();
+	while (!list_empty(&ifp->garp_delay->garp_list)) {
+		set_time_now();
+		if (timercmp(&time_now, &ifp->garp_delay->garp_next_time, <))
+			break;
 
-	list_for_each_entry(vrrp, &vrrp_data->vrrp, e_list) {
-		if (!vrrp->garp_pending && !vrrp->gna_pending)
-			continue;
+		ip_addr = list_first_entry(&ifp->garp_delay->garp_list, ip_address_t, garp_gna_list);
 
-		vrrp->garp_pending = false;
-		vrrp->gna_pending = false;
+		send_gratuitous_arp_immediate(ifp, ip_addr);
 
-		if (vrrp->state != VRRP_STATE_MAST || !vrrp->vipset)
-			continue;
-
-		vrrp_arpna_send(vrrp, &vrrp->vip, &next_time);
-		vrrp_arpna_send(vrrp, &vrrp->evip, &next_time);
+		list_del_init(&ip_addr->garp_gna_list);
+		if (--ip_addr->garp_gna_pending)
+			list_add_tail(&ip_addr->garp_gna_list, &ifp->garp_delay->garp_list);
 	}
 
-	if (next_time.tv_sec != INT_MAX) {
-		/* Register next timer tracker */
-		garp_next_time = next_time;
-		garp_thread = thread_add_timer(thread->master, vrrp_arp_thread, NULL,
-					       timer_long(timer_sub_now(next_time)));
-	} else
-		garp_thread = NULL;
+	if (!list_empty(&ifp->garp_delay->garp_list))
+		thread_add_timer(master, vrrp_arp_thread, ifp, timer_long(timer_sub_now(ifp->garp_delay->garp_next_time)));
+}
+
+/* Thread to send gratuitous NDs when the sending is rate limited */
+void
+vrrp_gna_thread(thread_ref_t thread)
+{
+	ip_address_t *ip_addr;
+	interface_t *ifp = THREAD_ARG(thread);
+
+	while (!list_empty(&ifp->garp_delay->gna_list)) {
+		set_time_now();
+		if (timercmp(&time_now, &ifp->garp_delay->gna_next_time, <))
+			break;
+
+		ip_addr = list_first_entry(&ifp->garp_delay->gna_list, ip_address_t, garp_gna_list);
+
+		ndisc_send_unsolicited_na_immediate(ifp, ip_addr);
+
+		list_del_init(&ip_addr->garp_gna_list);
+		if (--ip_addr->garp_gna_pending)
+			list_add_tail(&ip_addr->garp_gna_list, &ifp->garp_delay->gna_list);
+	}
+
+	if (!list_empty(&ifp->garp_delay->gna_list))
+		thread_add_timer(master, vrrp_gna_thread, ifp, timer_long(timer_sub_now(ifp->garp_delay->gna_next_time)));
 }
 
 #ifdef _WITH_DUMP_THREADS_
@@ -1480,7 +1473,7 @@ dump_threads(void)
 	set_time_now();
 	ctime_r(&time_now.tv_sec, time_buf);
 
-	fprintf(fp, "\n%.19s.%6.6ld: Thread dump\n", time_buf, time_now.tv_usec);
+	fprintf(fp, "\n%.19s.%6.6" PRI_tv_usec ": Thread dump\n", time_buf, time_now.tv_usec);
 
 	dump_thread_data(master, fp);
 
@@ -1489,7 +1482,7 @@ dump_threads(void)
 	fprintf(fp, "\n");
 	list_for_each_entry(vrrp, &vrrp_data->vrrp, e_list) {
 		ctime_r(&vrrp->sands.tv_sec, time_buf);
-		fprintf(fp, "VRRP instance %s, sands %.19s.%6.6ld, status %s\n", vrrp->iname, time_buf, vrrp->sands.tv_usec,
+		fprintf(fp, "VRRP instance %s, sands %.19s.%6.6" PRI_tv_usec ", status %s\n", vrrp->iname, time_buf, vrrp->sands.tv_usec,
 				vrrp->state == VRRP_STATE_INIT ? "INIT" :
 				vrrp->state == VRRP_STATE_BACK ? "BACKUP" :
 				vrrp->state == VRRP_STATE_MAST ? "MASTER" :
@@ -1505,9 +1498,12 @@ void
 register_vrrp_scheduler_addresses(void)
 {
 	register_thread_address("vrrp_arp_thread", vrrp_arp_thread);
+	register_thread_address("vrrp_gna_thread", vrrp_gna_thread);
 	register_thread_address("vrrp_dispatcher_init", vrrp_dispatcher_init);
 	register_thread_address("vrrp_gratuitous_arp_thread", vrrp_gratuitous_arp_thread);
 	register_thread_address("vrrp_lower_prio_gratuitous_arp_thread", vrrp_lower_prio_gratuitous_arp_thread);
+	register_thread_address("vrrp_gratuitous_arp_refresh_thread", vrrp_gratuitous_arp_refresh_thread);
+	register_thread_address("vrrp_gratuitous_arp_vmac_update_thread", vrrp_gratuitous_arp_vmac_update_thread);
 	register_thread_address("vrrp_script_child_thread", vrrp_script_child_thread);
 	register_thread_address("vrrp_script_thread", vrrp_script_thread);
 	register_thread_address("vrrp_read_dispatcher_thread", vrrp_read_dispatcher_thread);
